@@ -1,8 +1,9 @@
-/* A Bison parser, made by GNU Bison 3.0.4.  */
+/* A Bison parser, made by GNU Bison 3.4.  */
 
 /* Bison interface for Yacc-like parsers in C
 
-   Copyright (C) 1984, 1989-1990, 2000-2015 Free Software Foundation, Inc.
+   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2019 Free Software Foundation,
+   Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -30,8 +31,11 @@
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
-#ifndef YY_INI_HOME_PEA_POBJ_PHP_7_0_16_PHP_7_0_16_ZEND_ZEND_INI_PARSER_H_INCLUDED
-# define YY_INI_HOME_PEA_POBJ_PHP_7_0_16_PHP_7_0_16_ZEND_ZEND_INI_PARSER_H_INCLUDED
+/* Undocumented macros, especially those whose name start with YY_,
+   are private implementation details.  Do not rely on them.  */
+
+#ifndef YY_INI_ZEND_ZEND_INI_PARSER_H_INCLUDED
+# define YY_INI_ZEND_ZEND_INI_PARSER_H_INCLUDED
 /* Debug traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -90,4 +94,4 @@ typedef int YYSTYPE;
 
 int ini_parse (void);
 
-#endif /* !YY_INI_HOME_PEA_POBJ_PHP_7_0_16_PHP_7_0_16_ZEND_ZEND_INI_PARSER_H_INCLUDED  */
+#endif /* !YY_INI_ZEND_ZEND_INI_PARSER_H_INCLUDED  */
